<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\User;
use Illuminate\Http\Request;
use Spatie\Permission\Models\Role;
use Spatie\Permission\Models\Permission;
use Illuminate\Support\Facades\Hash;

class StaffController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth'); // Ensure only authenticated users access
        $this->middleware('role:Super Admin'); // Only Super Admin can access this
    }

    // List all staff members
    public function index()
    {
        $staff = User::role(['Admin', 'Manager', 'Editor'])->paginate(10);
        return view('admin.staff.index', compact('staff'));
    }

    // Show form to create a new staff member
    public function create()
    {
        $roles = Role::all();
        return view('admin.staff.create', compact('roles'));
    }

    // Store new staff member
    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required|string|max:255',
            'email' => 'required|email|unique:users,email',
            'password' => 'required|min:6|confirmed',
            'role' => 'required|exists:roles,name',
        ]);

        $staff = new User();
        $staff->name = $request->name;
        $staff->email = $request->email;
        $staff->password = Hash::make($request->password);
        $staff->assignRole($request->role);

        $staff->save();

        return redirect()->route('admin.staff.index')->with('success', 'Staff member created successfully.');
    }

    // Show edit form
    public function edit($id)
    {
        $staff = User::findOrFail($id);
        $roles = Role::all();
        return view('admin.staff.edit', compact('staff', 'roles'));
    }

    // Update staff member
    public function update(Request $request, $id)
    {
        $staff = User::findOrFail($id);

        $request->validate([
            'name' => 'required|string|max:255',
            'email' => 'required|email|unique:users,email,' . $id,
            'password' => 'nullable|min:6|confirmed',
            'role' => 'required|exists:roles,name',
        ]);

        $staff->name = $request->name;
        $staff->email = $request->email;
        if ($request->password) {
            $staff->password = Hash::make($request->password);
        }

        $staff->syncRoles([$request->role]);
        $staff->save();

        return redirect()->route('admin.staff.index')->with('success', 'Staff member updated successfully.');
    }

    // Delete staff member
    public function destroy($id)
    {
        $staff = User::findOrFail($id);
        $staff->delete();

        return redirect()->route('admin.staff.index')->with('success', 'Staff member deleted successfully.');
    }
}
