<?php

namespace App\Providers;

use App\Constants\Status;
use App\Lib\Searchable;
use App\Models\AdminNotification;
use App\Models\Deposit;
use App\Models\Frontend;
use App\Models\SupportTicket;
use App\Models\User;
use App\Models\Withdrawal;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\ServiceProvider;
use Illuminate\Pagination\Paginator;
use Illuminate\Support\Facades\View;
use Illuminate\Support\Facades\Schema;
use Illuminate\Support\Facades\URL;

class AppServiceProvider extends ServiceProvider
{
    /**
     * Register any application services.
     */
    public function register(): void
    {
        Builder::mixin(new Searchable);
    }

    /**
     * Bootstrap any application services.
     */
    public function boot(): void
    {
        // ✅ Fix for MySQL max key length error
        Schema::defaultStringLength(191);

        // ✅ Check System Installation Properly
        if (!config('app.installed')) {
            $envFilePath = base_path('.env');
            if (!file_exists($envFilePath) || empty(file_get_contents($envFilePath))) {
                redirect()->to('install')->send();
            } else {
                config(['app.installed' => true]);
            }
        }

        // ✅ Set Active Template
        $activeTemplate = activeTemplate();
        view()->share([
            'activeTemplate' => $activeTemplate,
            'activeTemplateTrue' => activeTemplate(true),
            'emptyMessage' => 'Data not found'
        ]);

        // ✅ Sidebar Data
        view()->composer('admin.partials.sidenav', function ($view) {
            $view->with([
                'bannedUsersCount' => User::banned()->count(),
                'emailUnverifiedUsersCount' => User::emailUnverified()->count(),
                'mobileUnverifiedUsersCount' => User::mobileUnverified()->count(),
                'kycUnverifiedUsersCount' => User::kycUnverified()->count(),
                'kycPendingUsersCount' => User::kycPending()->count(),
                'pendingTicketCount' => SupportTicket::whereIn('status', [Status::TICKET_OPEN, Status::TICKET_REPLY])->count(),
                'pendingDepositsCount' => Deposit::pending()->count(),
                'pendingWithdrawCount' => Withdrawal::pending()->count(),
                'updateAvailable' => version_compare(gs('available_version'), systemDetails()['version'], '>') ? 'v' . gs('available_version') : false,
            ]);
        });

        // ✅ Top Navigation Data
        view()->composer('admin.partials.topnav', function ($view) {
            $view->with([
                'adminNotifications' => AdminNotification::where('is_read', Status::NO)->with('user')->orderBy('id', 'desc')->take(10)->get(),
                'adminNotificationCount' => AdminNotification::where('is_read', Status::NO)->count(),
            ]);
        });

        // ✅ SEO Data
        view()->composer('partials.seo', function ($view) {
            $seo = Frontend::where('data_keys', 'seo.data')->first();
            $view->with([
                'seo' => $seo ? $seo->data_values : null,
            ]);
        });

        // ✅ Force HTTPS if Enabled
        if (gs('force_ssl')) {
            URL::forceScheme('https');
        }

        // ✅ Use Bootstrap Pagination
        Paginator::useBootstrapFive();
    }
}
